@@ -35,11 +35,11 @@ impl Overlay for EntryOverlay {
     }
 }
 impl EntryOverlay {
-    pub fn new(lang: &Language) -> Box<EntryOverlay> {
+    pub fn new(lang: Language) -> Box<EntryOverlay> {
         Box::new(EntryOverlay {
             capture_base: String::new(),
             overlay_type: "spec/overalys/entry/1.0".to_string(),
-            language: *lang,
+            language: lang,
             attr_entries: BTreeMap::new(),
         })
     }
